{% load i18n %}

<p>{% blocktrans trimmed %}Specify the details for the database backup.{% endblocktrans %}</p>
<p>{% blocktrans trimmed %}You can perform an incremental backup by specifying a parent backup. <strong>However,</strong> not all databases support incremental backups in which case this operation will result in an error.{% endblocktrans %}</p>
