{% load i18n %}

<p>
  {% blocktrans trimmed %}
  Move networks from 'Available Networks' to 'Selected Networks' by
  clicking the button, or dragging and dropping. You can change the
  NIC order by dragging and dropping as well.
  {% endblocktrans %}
</p>
