{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body-right %}
  <p>{% trans "Specify the new volume size for the database instance." %}</p>
  <p><strong>{% trans "Please note:</strong> The new value must be greater than the existing volume size." %}</strong></p>
{% endblock %}
